/*
Codan 8727 test 02
Make it a full frequ VFO for testing. No band switching.
*/
#define Version "TestVFO_01"

#include "si5351.h"
#include "Wire.h"
#include "LiquidCrystal_I2C.h"

// synthesiser default settings on start up.
unsigned long frequency =  10000000;          // 4 bytes required for frequency
unsigned long frequency0 =  0000000;          // 4 bytes required for frequency

//unsigned long frequency1 =  1810000;        // 4 bytes required for frequency
unsigned long frequency1  =  3525000;          // 4 bytes required for frequency  Bodge to make it go until coils fixed

unsigned long frequency2  =  7000000;          // 4 bytes required for frequency
unsigned long frequency3  =  7000000;          // 4 bytes required for frequency
unsigned long frequency4  = 10100000;          // 4 bytes required for frequency
unsigned long frequency5  = 14100000;          // 4 bytes required for frequency
unsigned long frequency6  = 18068000;          // 4 bytes required for frequency
unsigned long frequency7  = 21000000;          // 4 bytes required for frequency
unsigned long frequency8  = 24000000;          // 4 bytes required for frequency
unsigned long frequency9  = 28000000;          // 4 bytes required for frequency
unsigned long frequency10 = 28000000;          // 4 bytes required for frequency

unsigned long step_size = 1000;             // default tuning increment (Hz)
unsigned long offset = 1650000;             // IF offset for clockgen when driving Kestral/Hawk RFDS Radio
static signed char ham_band_index = 0;      // start on 0hz

unsigned long LSB =  1647000;           // carrier mixer for USB 0.01Hz res for Si5351 libs.
unsigned long USB =  1650000;           // carrier mixer for USB 0.01Hz res for Si5351 libs.

int32_t cal_factor = 16360; //  eg. 16360 for board found from Si5351 calibrator.

long encoder;                               // encoder direction: CCW=-1; NC=0; CW=1
Si5351 clockgen;                            // initalize clockgen

// encoder definitions
#define ENCODER_A  A0
#define ENCODER_B  A1
#define SSB_SWITCH A2                       // HIGH = LSB, LOW = USB.
// A4 = SDA
// A5 = SCL
#define BAND_SENSE     A6                  //
#define ENCODER_BUTTON 5                  // selects tuning decade
#define Output1  13                       // not used yet
                           
#define ENCODER_INPUT_PORT PINC
int sideband = 0;   
int last_sideband = 1;

/* HF Ham bands
Band        UK Allocation          USA Allocation    Australia Allocation
(metres)          Mhz                   Mhz                 Mhz
 160         1.810 -  2.000        1.800 -  2.000        1.800 -  1.875
  80         3.500 -  3.800        3.500 -  4.000        3.500 -  3.800
  40         7.000 -  7.200        7.000 -  7.300        7.000 -  7.300
  30        10.100 - 10.150       10.100 - 10.150       10.100 - 10.150
  20        14.100 - 14.350       14.100 - 14.350       14.100 - 14.350
  17        18.068 - 18.168       18.068 - 18.168       18.068 - 18.168
  15        21.000 - 21.450       21.000 - 21.450       21.000 - 21.450 
  12        24.890 - 24.990       24.890 - 24.990       24.890 - 24.990
  10        28.000 - 29.700       28.000 - 29.700       28.000 - 29.700 
 */
LiquidCrystal_I2C lcd(0x27,20,4);  // set the LCD address to 0x30 maybe??

void setup() { 

  lcd.init();                      // initialize the lcd 
  lcd.backlight();     
  lcd.print(Version);
  delay(1000);
  lcd.clear();
 
  // Initialize the Si5351
  clockgen.init(SI5351_CRYSTAL_LOAD_8PF, 0, 0);
  clockgen.set_correction(cal_factor, SI5351_PLL_INPUT_XO); // <<<<<<<this is found from Si5351 calibrator code.

  // configure rotary encoder & push-button
  pinMode(ENCODER_A, INPUT_PULLUP);     // encoder leads
  pinMode(ENCODER_B, INPUT_PULLUP);  
  pinMode(ENCODER_BUTTON, INPUT_PULLUP);
  pinMode(SSB_SWITCH, INPUT_PULLUP);    // Hi = LSB, LOW =  USB
  pinMode(Output1,OUTPUT);
  digitalWrite(Output1,HIGH);     // output1 off

  display_freq_step();            // display frequency & step-size
  set_synth();
}

/******************************
 main loop
*******************************/
void loop() {
  check_band();
  check_step_size();
  
if (digitalRead(SSB_SWITCH)!=last_sideband)
    { set_synth();
      display_freq_step();                        // display frequency & step-size
    }
  // check the frequency 
  encoder = read_encoder();                       // returns +1/0/-1 (cw/nc/ccw)
  if (encoder != 0) {
    frequency = frequency + (encoder * step_size);
    
   set_synth();
    display_freq_step();                          // display frequency & step-size
  } 
}

/******************************
 read encoder direction
*******************************/
char read_encoder() {
  // definitions
  #define DELAY 100                               // number of polling-loops   
  static unsigned short integrator;               // encoder debounce
 
  /* the array values are determined by combining the previous and current A1,A0 
  Gray Code bit-patterns to form an array-index. +1 is assigned to all clockwise 
  (CW) indexes; -1 is assigned to all counter-clockwise (CCW) indexes; and 0 is 
  assigned to the remaining positions to indicate no movement*/   
  static char encoder[] = {0,-1,1,0,1,0,0,-1,-1,0,0,1,0,1,-1,0}; // SWAPPED PHASES
  //   static char encoder[] = {0,1,-1,0,-1,0,0,1,1,0,0,-1,0,-1,1,0};
  static unsigned char encoder_index = 0;
  
  static long count = 0;                         // CW rotation = +1; CCW rotation = -1
  static long this_reading;
  static long last_reading;
  char value;                                    // return value

  /* check for encoder rotation. The absolute "count" value changes as the encoder rotates.
  Typically there are four states (counts) per click, but often more due to contact bounce.
  Bounce doesn't matter so long as the "count" decreases" for CCW rotation; remains constant 
  when there is no rotation; and increases with CW rotation. Bounce can be completely 
  eliminated if we look for the "detent" code and integrate as shown below. */  
  encoder_index <<= 2;                           // shift previous bit-pattern left two places
  encoder_index |= (ENCODER_INPUT_PORT & 0x03);  // get current bit-pattern and combine with previous
  count += encoder[( encoder_index & 0x0f )];     
  
  // debounce encoder
  if ((ENCODER_INPUT_PORT & 0x03) == 3) {        // encoder on an indent (3 decimal == 00000011 binary)
    if (integrator < DELAY) {                    
      integrator++;
    } 
  } else {                                       // encoder not resting on indent
     if (integrator > 0) {
       integrator--;
     }
  }
  
  // prepare the "return" value
  if (integrator >= DELAY) {                     // encoder deemed to be stationary
    this_reading = count;
    if (this_reading < last_reading) value = -1;
    if (this_reading == last_reading) value = 0;
    if (this_reading > last_reading) value = 1;
    last_reading = this_reading;                 // both readings now hold current "count"
    integrator = 0;                              // reset integrator
  } else {
    value = 0;
  }
  return(value);
}
  
/******************************
 check frequency band
*******************************/
void check_band() {               // band now set from A6 reading
  int temp = 0;
  temp = (analogRead(BAND_SENSE)/60);
    
if (ham_band_index!=temp){    // update the band if it has changed
        ham_band_index=temp;
  long ham_band[] = {frequency0,frequency1,frequency2,frequency3,frequency4,frequency5,frequency6,frequency7,frequency8,frequency9,frequency10};
        frequency = ham_band[ham_band_index];     // change frequency band  
        set_synth ();                             // set the new frequ ****** added 20190502
        display_freq_step(); 
}
}


/******************************
 check the step size
*******************************/
void check_step_size() {
  // delay definitions
  #define DELAY 3000                             // DELAY = debounce(mS)*loops/second
  #define LONG_PUSH 15000                        // number of loops considered "long"
  
  // variables
  static boolean transition = false;             // valid during button push
  static unsigned short integrator;              // varies from 0 to MAXIMUM
  static unsigned short push_duration;           // determines step size/direction
  
  // look for first high-to-low transition
  if (!transition && (digitalRead(ENCODER_BUTTON) == LOW)) {
    transition = true;
    integrator = 0;
    push_duration = 0;
  }
  
  // look for last low-to-high transition
  if (transition) {
    if (push_duration < (LONG_PUSH*2)) {         // track push duration
      push_duration++;
    }
    if (digitalRead(ENCODER_BUTTON) == HIGH) {   // ENCODER_BUTTON is HIGH
      if (integrator < DELAY) {        
        integrator++;
      } else {                                   // ENCODER_BUTTON is LOW
        if (integrator > 0) {
    integrator--;
  }
      }
    }
   
    // update the step_size
    if (integrator >= DELAY) { 
      transition = false;
      if (push_duration > LONG_PUSH) {          // LONG PUSH: decrease step size
        step_size = step_size/10;
        if (step_size < 1) {                    // was 10
          step_size = 100000;                   // wrap around    
        }        
      } else {                                  // SHORT PUSH: increase step size
        step_size = step_size*10;
        if (step_size > 100000) {
          step_size = 1;                       // wrap around Was 10
        }         
      }     
      display_freq_step();                      // update step-size     
    }
  }
}

/*******************************
set the Si5351 CLK0 frequency
 ******************************/
void set_synth ()
{
  /*
switch (ham_band_index) {       // limit to SSB part of bands

    case 0:
  frequency = constrain(frequency, 00000000, 00000000);
  frequency0 = frequency;
  break;

    case 1:
  frequency = constrain(frequency, 1840000, 1875000);
  frequency1 = frequency;
  break;

      case 2:
  frequency = constrain(frequency, 3525000, 3700000);
  frequency2 = frequency;
  break; 

    case 3:
  frequency = constrain(frequency, 7040000, 7300000);
  frequency3 = frequency;
  break;

    case 4:
  frequency = constrain(frequency, 10115000, 10140000);
  frequency4 = frequency;
  break;

    case 5:
  frequency = constrain(frequency, 14112000, 14350000);
  frequency5 = frequency;
  break;

    case 6:
  frequency = constrain(frequency, 18110000, 18168000);
  frequency6 = frequency;
  break;

    case 7:
  frequency = constrain(frequency, 21150000, 21450000);
  frequency7 = frequency;
  break;

    case 8:
  frequency = constrain(frequency, 10000000, 100000000);
  frequency8 = frequency;
  break;
  
    case 9:
  frequency = constrain(frequency, 00000000, 00000000);
  frequency9 = frequency;
  break;
  
     case 10:
  frequency = constrain(frequency, 00000000, 00000000);
  frequency10 = frequency;
  break;
 
}
*/

  frequency = constrain(frequency, 1000000, 16000000);



  
    if (digitalRead(SSB_SWITCH)==LOW) // USB
    {
      clockgen.set_freq(USB*100, SI5351_CLK1);
    last_sideband=0;
    }
  else                                // LSB
    {
      clockgen.set_freq(LSB*100, SI5351_CLK1);
    last_sideband=1;
    }
    // set the frequency
    clockgen.set_freq(((frequency+offset)*100), SI5351_CLK0);
    last_sideband=1;
    clockgen.set_freq(((frequency)*100), SI5351_CLK2);
}

 
/********************************
 update frequency and step-size
********************************/
void display_freq_step() {

   


  // display frequency
  lcd.clear();
        lcd.setCursor(0,2); 
      if (digitalRead(SSB_SWITCH)==HIGH)
      { lcd.print("LSB ");                          
        show_Mhz(LSB);  
      } 
      else 
      { lcd.print("USB ");                         
        show_Mhz(USB); 
      }
     lcd.print(" CK1");  
     lcd.setCursor(0,0) ;
     lcd.print("F+O");    

    lcd.setCursor(4,0); 
    show_Mhz ( frequency+offset); 
    lcd.print(" CK0");  

    lcd.setCursor(0,3); 
    lcd.print("Sig ");
    show_Mhz ( frequency); 
    lcd.print(" CK2");  

  // display step size
  switch (step_size) {
    
      case 1:                           // added for 1xHz.
      lcd.setCursor(12,1);
      lcd.print("*");       
      break;     
    case 10:
      lcd.setCursor(11,1);
      lcd.print("*");       
      break;
    case 100:
      lcd.setCursor(10,1);
      lcd.print("*");       
      break;
    case 1000:
      lcd.setCursor(9,1);
      lcd.print("*");       
      break;
    case 10000:
      lcd.setCursor(8,1);
      lcd.print("*");       
      break;
    case 100000:
      lcd.setCursor(7,1);
      lcd.print("*");       
      break;
    default:
      break;

  }
}

void show_Mhz (long frqx )
{
  // convert frequency to floating point

  double Mhz = (double) frqx/1000000;
  if (Mhz < 10) {
    lcd.print(" ");                         
  } lcd.print(Mhz, 6);                          // 6 digit resolution
    lcd.print("MHz");
  
}
